@@ -1,19 +1,20 @@
 import re
 from datetime import datetime
-from typing import Dict, List, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Union, cast
 
 import cognite.client.utils._time
 from cognite.client import utils
 from cognite.client._api_client import APIClient
 from cognite.client.data_classes import Datapoints, DatapointsList, TimeSeries
-from cognite.client.exceptions import CogniteAPIError
-from cognite.client.utils._experimental_warning import experimental_api
+
+if TYPE_CHECKING:
+    import sympy
 
 
 class SyntheticDatapointsAPI(APIClient):
     _RESOURCE_PATH = "/timeseries/synthetic"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
         self._DPS_LIMIT = 10000
 
@@ -65,13 +66,13 @@ class SyntheticDatapointsAPI(APIClient):
                 >>> dps = c.datapoints.synthetic.query([sin(a), cos(a)], start="2w-ago", end="now", variables={"a": "my_ts_external_id"}, aggregate='interpolation', granularity='1m')
         """
         if limit is None or limit == -1:
-            limit = float("inf")
+            limit = cast(int, float("inf"))
 
         tasks = []
         expressions_to_iterate = expressions if isinstance(expressions, List) else [expressions]
 
         for i in range(len(expressions_to_iterate)):
-            expression, short_expression = SyntheticDatapointsAPI._build_expression(
+            expression, short_expression = self._build_expression(
                 expressions_to_iterate[i], variables, aggregate, granularity
             )
             query = {
@@ -97,7 +98,7 @@ class SyntheticDatapointsAPI(APIClient):
             else datapoints_summary.results[0]
         )
 
-    def _fetch_datapoints(self, query, datapoints, limit):
+    def _fetch_datapoints(self, query: Dict[str, Any], datapoints: Datapoints, limit: int) -> Datapoints:
         while True:
             query["limit"] = min(limit, self._DPS_LIMIT)
             resp = self._post(url_path=self._RESOURCE_PATH + "/query", json={"items": [query]})
@@ -109,45 +110,53 @@ class SyntheticDatapointsAPI(APIClient):
             query["start"] = data["datapoints"][-1]["timestamp"] + 1
         return datapoints
 
-    @staticmethod
-    def _build_expression(expression, variables=None, aggregate=None, granularity=None):
+    @classmethod
+    def _build_expression(
+        cls,
+        expression: Union[str, "sympy.Expr"],
+        variables: Dict[str, Any] = None,
+        aggregate: str = None,
+        granularity: str = None,
+    ) -> Tuple[str, str]:
         if expression.__class__.__module__.startswith("sympy."):
-            expression = SyntheticDatapointsAPI._sympy_to_sts(expression)
+            expression_str = cls._sympy_to_sts(expression)
             if not variables:
                 raise ValueError(
                     "sympy expressions are only supported in combination with the `variables` parameter to map symbols to time series."
                 )
+        else:
+            expression_str = cast(str, expression)
         if aggregate and granularity:
             aggregate_str = ",aggregate:'{}',granularity:'{}'".format(aggregate, granularity)
         else:
             aggregate_str = ""
-        expression_with_ts = expression
+        expression_with_ts: str = expression_str
         if variables:
             for k, v in variables.items():
                 if isinstance(v, TimeSeries):
                     v = v.external_id
-                expression_with_ts = re.sub(
+                expression_with_ts = re.sub(  # type: ignore
                     re.compile(r"\b%s\b" % k), "ts{externalId:'%s'%s}" % (v, aggregate_str), expression_with_ts
                 )
-        return expression_with_ts, expression
+        return expression_with_ts, expression_str
 
     @staticmethod
-    def _sympy_to_sts(expression):
-        sympy = utils._auxiliary.local_import("sympy")
+    def _sympy_to_sts(expression: Union[str, "sympy.Expr"]) -> str:
+        sympy_module = cast(Any, utils._auxiliary.local_import("sympy"))
 
-        infix_ops = {sympy.Add: "+", sympy.Mul: "*"}
+        infix_ops = {sympy_module.Add: "+", sympy_module.Mul: "*"}
         functions = {
-            sympy.cos: "cos",
-            sympy.sin: "sin",
-            sympy.sqrt: "sqrt",
-            sympy.log: "ln",
-            sympy.exp: "exp",
-            sympy.Abs: "abs",
+            sympy_module.cos: "cos",
+            sympy_module.sin: "sin",
+            sympy_module.sqrt: "sqrt",
+            sympy_module.log: "ln",
+            sympy_module.exp: "exp",
+            sympy_module.Abs: "abs",
         }
 
-        def process_symbol(sym):
-            if isinstance(sym, sympy.AtomicExpr):
-                if isinstance(sym, sympy.NumberSymbol):
+        def process_symbol(sym: Any) -> str:
+            if isinstance(sym, sympy_module.AtomicExpr):
+                if isinstance(sym, sympy_module.NumberSymbol):
                     return str(sym.evalf(15))
                 else:
                     return str(sym)
@@ -155,7 +164,7 @@ class SyntheticDatapointsAPI(APIClient):
             infixop = infix_ops.get(sym.__class__)
             if infixop:
                 return "(" + infixop.join(process_symbol(s) for s in sym.args) + ")"
-            if isinstance(sym, sympy.Pow):
+            if isinstance(sym, sympy_module.Pow):
                 if sym.args[1] == -1:
                     return "(1/{})".format(process_symbol(sym.args[0]))
                 return "pow({},{})".format(*[process_symbol(x) for x in sym.args])
