@@ -1,11 +1,31 @@
-from cognite.client.data_classes._base import *
+from typing import TYPE_CHECKING, Any, Awaitable, Dict, List, Optional, Union, cast
+
+from cognite.client import utils
+from cognite.client.data_classes._base import (
+    CogniteFilter,
+    CognitePrimitiveUpdate,
+    CogniteResource,
+    CogniteResourceList,
+    CogniteUpdate,
+)
+from cognite.client.data_classes.iam import ClientCredentials
 from cognite.client.data_classes.shared import TimestampRange
 from cognite.client.data_classes.transformations._alphatypes import AlphaDataModelInstances
-from cognite.client.data_classes.transformations.common import *
+from cognite.client.data_classes.transformations.common import (
+    NonceCredentials,
+    OidcCredentials,
+    RawTable,
+    SequenceRows,
+    TransformationBlockedInfo,
+    TransformationDestination,
+)
 from cognite.client.data_classes.transformations.jobs import TransformationJob, TransformationJobList
 from cognite.client.data_classes.transformations.schedules import TransformationSchedule
 from cognite.client.data_classes.transformations.schema import TransformationSchemaColumnList
 
+if TYPE_CHECKING:
+    from cognite.client import CogniteClient
+
 
 class Transformation(CogniteResource):
     """The transformations resource allows transforming data in CDF.
@@ -65,7 +85,9 @@ class Transformation(CogniteResource):
         blocked: TransformationBlockedInfo = None,
         schedule: "TransformationSchedule" = None,
         data_set_id: int = None,
-        cognite_client=None,
+        cognite_client: "CogniteClient" = None,
+        source_nonce: Optional[NonceCredentials] = None,
+        destination_nonce: Optional[NonceCredentials] = None,
     ):
         self.id = id
         self.external_id = external_id
@@ -94,12 +116,87 @@ class Transformation(CogniteResource):
         self.blocked = blocked
         self.schedule = schedule
         self.data_set_id = data_set_id
-        self._cognite_client = cognite_client
+        self.source_nonce = source_nonce
+        self.destination_nonce = destination_nonce
+        self._cognite_client = cast("CogniteClient", cognite_client)
+
+    def copy(self) -> "Transformation":
+        return Transformation(
+            self.id,
+            self.external_id,
+            self.name,
+            self.query,
+            self.destination,
+            self.conflict_mode,
+            self.is_public,
+            self.ignore_null_fields,
+            self.source_api_key,
+            self.destination_api_key,
+            self.source_oidc_credentials,
+            self.destination_oidc_credentials,
+            self.created_time,
+            self.last_updated_time,
+            self.owner,
+            self.owner_is_current_user,
+            self.has_source_api_key,
+            self.has_destination_api_key,
+            self.has_source_oidc_credentials,
+            self.has_destination_oidc_credentials,
+            self.running_job,
+            self.last_finished_job,
+            self.blocked,
+            self.schedule,
+            self.data_set_id,
+            None,
+            self.source_nonce,
+            self.destination_nonce,
+        )
+
+    def _process_credentials(self, sessions_cache: Dict[str, NonceCredentials] = None, keep_none: bool = False) -> None:
+        if sessions_cache is None:
+            sessions_cache = {}
+
+        def try_get_or_create_nonce(oidc_credentials: Optional[OidcCredentials]) -> Optional[NonceCredentials]:
+            if keep_none and oidc_credentials is None:
+                return None
+
+            # MyPy requires this to make sure it's not changed to None after inner declaration
+            assert sessions_cache is not None
+
+            key = (
+                f"{oidc_credentials.client_id}:{oidc_credentials.client_secret.__hash__()}"
+                if oidc_credentials
+                else "DEFAULT"
+            )
+
+            ret = sessions_cache.get(key)
+            if not ret:
+                if oidc_credentials and oidc_credentials.client_id and oidc_credentials.client_secret:
+                    credentials = ClientCredentials(oidc_credentials.client_id, oidc_credentials.client_secret)
+                else:
+                    credentials = None
+                try:
+                    session = self._cognite_client.iam.sessions.create(credentials)
+                    ret = NonceCredentials(session.id, session.nonce, self._cognite_client._config.project)
+                    sessions_cache[key] = ret
+                except Exception:
+                    ret = None
+            return ret
+
+        if self.source_api_key is None and self.source_nonce is None:
+            self.source_nonce = try_get_or_create_nonce(self.source_oidc_credentials)
+            if self.source_nonce:
+                self.source_oidc_credentials = None
+
+        if self.destination_api_key is None and self.destination_nonce is None:
+            self.destination_nonce = try_get_or_create_nonce(self.destination_oidc_credentials)
+            if self.destination_nonce:
+                self.destination_oidc_credentials = None
 
     def run(self, wait: bool = True, timeout: Optional[float] = None) -> "TransformationJob":
         return self._cognite_client.transformations.run(transformation_id=self.id, wait=wait, timeout=timeout)
 
-    def cancel(self):
+    def cancel(self) -> None:
         if self.id is None:
             self._cognite_client.transformations.cancel(transformation_external_id=self.external_id)
         else:
@@ -112,14 +209,14 @@ class Transformation(CogniteResource):
         return self._cognite_client.transformations.jobs.list(transformation_id=self.id)
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client=None):
+    def _load(cls, resource: Union[Dict, str], cognite_client: "CogniteClient" = None) -> "Transformation":
         instance = super(Transformation, cls)._load(resource, cognite_client)
         if isinstance(instance.destination, Dict):
             snake_dict = {utils._auxiliary.to_snake_case(key): value for (key, value) in instance.destination.items()}
             if instance.destination.get("type") == "raw":
                 snake_dict.pop("type")
                 instance.destination = RawTable(**snake_dict)
-            elif instance.destination.get("type") == "data_model_instances":
+            elif instance.destination.get("type") == "alpha_data_model_instances":
                 snake_dict.pop("type")
                 instance.destination = AlphaDataModelInstances(**snake_dict)
             elif instance.destination.get("type") == "sequence_rows":
@@ -154,17 +251,24 @@ class Transformation(CogniteResource):
             Dict[str, Any]: A dictionary representation of the instance.
         """
         ret = super().dump(camel_case=camel_case)
-        if self.source_oidc_credentials:
+        if isinstance(self.source_oidc_credentials, OidcCredentials):
             source_key = "sourceOidcCredentials" if camel_case else "source_oidc_credentials"
             ret[source_key] = self.source_oidc_credentials.dump(camel_case=camel_case)
-        if self.destination_oidc_credentials:
+        if isinstance(self.destination_oidc_credentials, OidcCredentials):
             destination_key = "destinationOidcCredentials" if camel_case else "destination_oidc_credentials"
             ret[destination_key] = self.destination_oidc_credentials.dump(camel_case=camel_case)
+
+        if isinstance(self.source_nonce, NonceCredentials):
+            destination_key = "sourceNonce" if camel_case else "source_nonce"
+            ret[destination_key] = self.source_nonce.dump(camel_case=camel_case)
+        if isinstance(self.destination_nonce, NonceCredentials):
+            destination_key = "destinationNonce" if camel_case else "destination_nonce"
+            ret[destination_key] = self.destination_nonce.dump(camel_case=camel_case)
         if isinstance(self.destination, AlphaDataModelInstances) or isinstance(self.destination, SequenceRows):
             ret["destination"] = self.destination.dump(camel_case=camel_case)
         return ret
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(self.external_id)
 
 
@@ -181,54 +285,54 @@ class TransformationUpdate(CogniteUpdate):
             return self._set(value)
 
     @property
-    def external_id(self):
+    def external_id(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "externalId")
 
     @property
-    def name(self):
+    def name(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "name")
 
     @property
-    def destination(self):
+    def destination(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "destination")
 
     @property
-    def conflict_mode(self):
+    def conflict_mode(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "conflictMode")
 
     @property
-    def query(self):
+    def query(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "query")
 
     @property
-    def source_oidc_credentials(self):
+    def source_oidc_credentials(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "sourceOidcCredentials")
 
     @property
-    def destination_oidc_credentials(self):
+    def destination_oidc_credentials(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "destinationOidcCredentials")
 
     @property
-    def source_api_key(self):
+    def source_api_key(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "sourceApiKey")
 
     @property
-    def destination_api_key(self):
+    def destination_api_key(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "destinationApiKey")
 
     @property
-    def is_public(self):
+    def is_public(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "isPublic")
 
     @property
-    def ignore_null_fields(self):
+    def ignore_null_fields(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "ignoreNullFields")
 
     @property
-    def data_set_id(self):
+    def data_set_id(self) -> _PrimitiveTransformationUpdate:
         return TransformationUpdate._PrimitiveTransformationUpdate(self, "dataSetId")
 
-    def dump(self, camel_case: bool = True):
+    def dump(self, camel_case: bool = True) -> Dict[str, Any]:
         obj = super().dump()
         dest = obj.get("update", {}).get("destination", {}).get("set")
         if isinstance(dest, AlphaDataModelInstances) or isinstance(dest, SequenceRows):
@@ -238,7 +342,6 @@ class TransformationUpdate(CogniteUpdate):
 
 class TransformationList(CogniteResourceList):
     _RESOURCE = Transformation
-    _UPDATE = TransformationUpdate
 
 
 class TransformationFilter(CogniteFilter):
@@ -251,7 +354,7 @@ class TransformationFilter(CogniteFilter):
         destination_type (str): Transformation destination resource name to filter by.
         conflict_mode (str): Filters by a selected transformation action type: abort/create, upsert, update, delete
         cdf_project_name (str): Project name to filter by configured source and destination project
-        has_blocked_error (str): Whether only the blocked transformations should be included in the results.
+        has_blocked_error (bool): Whether only the blocked transformations should be included in the results.
         created_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps
         last_updated_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps
         data_set_ids (List[Dict[str, Any]]): Return only transformations in the specified data sets with these ids.
@@ -282,8 +385,8 @@ class TransformationFilter(CogniteFilter):
         self.data_set_ids = data_set_ids
 
     @classmethod
-    def _load(self, resource: Union[Dict, str], cognite_client=None):
-        instance = super(TransformationFilter, self)._load(resource, cognite_client)
+    def _load(self, resource: Union[Dict, str]) -> "TransformationFilter":
+        instance = super(TransformationFilter, self)._load(resource)
         if isinstance(resource, Dict):
             if instance.created_time is not None:
                 instance.created_time = TimestampRange(**instance.created_time)
@@ -291,7 +394,7 @@ class TransformationFilter(CogniteFilter):
                 instance.last_updated_time = TimestampRange(**instance.last_updated_time)
         return instance
 
-    def dump(self, camel_case: bool = True):
+    def dump(self, camel_case: bool = True) -> Dict[str, Any]:
         obj = super().dump(camel_case=camel_case)
         if obj.get("includePublic"):
             is_public = obj.pop("includePublic")
@@ -308,14 +411,17 @@ class TransformationPreviewResult(CogniteResource):
     """
 
     def __init__(
-        self, schema: "TransformationSchemaColumnList" = None, results: List[Dict] = None, cognite_client=None
-    ):
+        self,
+        schema: "TransformationSchemaColumnList" = None,
+        results: List[Dict] = None,
+        cognite_client: "CogniteClient" = None,
+    ) -> None:
         self.schema = schema
         self.results = results
-        self._cognite_client = cognite_client
+        self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: Union[Dict, str], cognite_client=None):
+    def _load(cls, resource: Union[Dict, str], cognite_client: "CogniteClient" = None) -> "TransformationPreviewResult":
         instance = super(TransformationPreviewResult, cls)._load(resource, cognite_client)
         if isinstance(instance.schema, Dict):
             items = instance.schema.get("items")
